﻿using System;
using System.Text;
using System.Threading;
using ximc;

namespace testcs
{
    class testapp
    {
        private static API.LoggingCallback callback;
        private static void MyLog (API.LogLevel loglevel, string message)
        {
            Console.WriteLine("MyLog {0}: {1}", loglevel, message);
        }

        private static void print_status(status_t status)
        {
            Console.WriteLine("rpm: {0} pos: {1} flags: {2}",
                        status.CurSpeed, status.CurPosition, status.Flags);
        }

        private static void print_status_calb(status_calb_t status_calb)
        {
            Console.WriteLine("speed: {0} {3}/s position: {1} {3} flags: {2}",
                        status_calb.CurSpeed, status_calb.CurPosition, status_calb.Flags, "mm");
        }

        static void Main(string[] args)
        {
            int device = -1;
            Result res;
            try
            {
                callback = new API.LoggingCallback(MyLog);
                API.set_logging_callback(callback);
                // Pointer to device enumeration structure
                IntPtr device_enumeration;
                // Enumerates all devices
                device_enumeration = API.enumerate_devices(0);
                if (device_enumeration == null)
                    throw new Exception("Error enumerating devices");

                // Gets found devices count
                int device_count = API.get_device_count(device_enumeration);
                if (device_count == 0)
                    throw new Exception("No devices");

                // List all found devices
                for (int i = 0; i < device_count; ++i)
                {
                    // Gets device name 
                    String dev = API.get_device_name(device_enumeration, i);
                    System.Console.WriteLine("Found device {0}", dev);
                }
		
                //String deviceName = "\\\\.\\COM8";
                // Get first device name
                String deviceName = API.get_device_name(device_enumeration, 0);
                System.Console.WriteLine("Using device {0}", deviceName);

                // Open this device
                device = API.open_device(deviceName);
                Console.WriteLine("Device {0}", device);

                status_t status;
                // Read device status
                res = API.get_status(device, out status);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());
                print_status(status);

                // Send "zero" command to device
                res = API.command_zero(device);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());
                Thread.Sleep(2);

                // Read device status
                res = API.get_status(device, out status);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());
                print_status(status);

                Console.WriteLine("running...");
                // Send "move to the right" command to the device
                res = API.command_right(device);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());

                Thread.Sleep(1 * 1000);
                // Read device status
                res = API.get_status(device, out status);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());
                print_status(status);

                Thread.Sleep(1 * 1000);
                status_calb_t status_calb;
                calibration_t calibration;
                engine_settings_t engine_settings;
                res = API.get_engine_settings(device, out engine_settings);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());
				
                calibration.A = 0.1;
                calibration.MicrostepMode = engine_settings.MicrostepMode;
                // Read calibrated device status
                res = API.get_status_calb(device, out status_calb, calibration);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());
                print_status_calb(status_calb);

                Thread.Sleep(1 * 1000);

                Console.WriteLine("stopping...");
                // Send "stop" command to the device
                res = API.command_stop(device);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());

                // Read device status
                res = API.get_status(device, out status);
                if (res != Result.ok)
                    throw new Exception("Error " + res.ToString());
                print_status(status);

                Console.WriteLine("Done...");
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception " + e.Message);
                Console.WriteLine(e.StackTrace.ToString());
            }
            finally
            {
                if (device != -1)
                    API.close_device(ref device);
            }
            //Console.ReadKey();
        }
    }
}
