% MATLAB test for XIMC library
% Variables for tested R2013a 64-bit/Windows7
lib='..\..\ximc\win64\libximc.dll';
inc='..\..\ximc\win64\ximc.h';

if not(libisloaded('libximc'))
    disp('Loading library')
    [notfound,warnings] = loadlibrary(lib,inc, 'alias', 'libximc');
end

device_names = ximc_enumerate_devices_wrap(0);
devices_count = size(device_names,2);
if devices_count == 0
    disp('No devices found')
    return
end
for i=1:devices_count
    disp(['Found device: ', device_names{1,i}]);
end
device_name = device_names{1,1};
disp(['Using device name ', device_name]);

device_id = calllib('libximc','open_device', device_name);
disp(['Using device id ', num2str(device_id)]);

state_s = ximc_get_status(device_id);
disp('Status:'); disp(state_s);

disp('Zeroing engine...');
result = calllib('libximc','command_zero', device_id);
if result ~= 0
    disp(['Command failed with code', num2str(result)]);
end

disp('Running engine to the right for 5 seconds...');
result = calllib('libximc','command_right', device_id);
if result ~= 0
    disp(['Command failed with code', num2str(result)]);
end

pause(2);
calb = struct();
calb.A = 0.1; % arbitrary choice for example, set by user in real scenarios
calb.MicrostepMode = 9; % == MICROSTEP_MODE_FRAC_256
state_calb_s = ximc_get_status_calb(device_id, calb);
disp('Status calb:'); disp(state_calb_s);

pause(1);

state_s = ximc_get_status(device_id);
disp('Status:'); disp(state_s);

pause(2);

state_s = ximc_get_status(device_id);
disp('Status:'); disp(state_s);

disp('Stopping engine...');
result = calllib('libximc','command_stop', device_id);
if result ~= 0
    disp(['Command failed with code', num2str(result)]);
end

state_s = ximc_get_status(device_id);
disp('Status:'); disp(state_s);

device_id_ptr = libpointer('int32Ptr', device_id);
calllib('libximc','close_device', device_id_ptr);
disp('Done');
% when needed
% unloadlibrary('libximc');
